@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 #
 # A Python Gtk UI for Hatari that can embed the Hatari emulator window.
 #
@@ -311,7 +311,7 @@ class UICallbacks:
 
     # ------ snapshot load/save callbacks ---------
     def load(self, widget):
-        path = os.path.expanduser("~/.hatari/hatari.sav")
+        path = os.path.expanduser("~/.config/hatari/hatari.sav")
         filename = get_open_filename("Select snapshot", self.mainwin, path)
         if filename:
             self.memstate = ["--memstate", filename]
@@ -439,12 +439,15 @@ class UIActions:
         ("manual", None, "Hatari manual", None, None, self.help.view_hatari_manual),
         ("compatibility", None, "Hatari compatibility list", None, None, self.help.view_hatari_compatibility),
         ("release", None, "Hatari release notes", None, None, self.help.view_hatari_releasenotes),
+        ("uirelease", None, "Hatari UI release notes", None, None, self.help.view_hatariui_releasenotes),
         ("todo", None, "Hatari TODO", None, None, self.help.view_hatari_todo),
-        ("mails", None, "Hatari mailing lists", None, None, self.help.view_hatari_mails),
-        ("changes", None, "Latest Hatari changes", None, None, self.help.view_hatari_repository),
-        ("authors", None, "Hatari authors", None, None, self.help.view_hatari_authors),
+
         ("hatari", None, "Hatari home page", None, None, self.help.view_hatari_page),
         ("hatariui", None, "Hatari UI home page", None, None, self.help.view_hatariui_page),
+        ("mails", None, "Hatari mailing lists", None, None, self.help.view_hatari_mails),
+        ("changes", None, "Latest Hatari changes", None, None, self.help.view_hatari_repository),
+
+        ("authors", None, "Hatari authors", None, None, self.help.view_hatari_authors),
         ("about", Gtk.STOCK_INFO, "Hatari UI info", "<Ctrl>I", "Hatari UI information", cb.about)
         ))
         self.action_names = [x.get_name() for x in self.actions.list_actions()]
@@ -617,7 +620,7 @@ class UIActions:
         ("Devices", ("display", "floppy", "harddisk", "joystick", "machine", "device", "sound")),
         ("Configuration", ("path", None, "lconfig", "sconfig")),
         ("Debug", ("debug", "trace")),
-        ("Help", ("manual", "compatibility", "release", "todo", None, "mails", "changes", None, "authors", "hatari", "hatariui", "about",))
+        ("Help", ("manual", "compatibility", "release", "uirelease", "todo", None, "hatari", "hatariui", "mails", "changes", None, "authors", "about",))
         )
         bar = Gtk.MenuBar()
 
@@ -672,7 +675,7 @@ def usage(actions, msg=None):
     print("\nOptions:")
     print("\t-h, --help\t\tthis help")
     print("\t-n, --nomenu\t\tomit menus")
-    print("\t-e, --embed\t\tembed Hatari window in middle of controls")
+    print("\t-e, --embed\t\tembed Hatari window in middle of controls (X11 only)")
     print("\t-f, --fullscreen\tstart in fullscreen")
     print("\t-l, --left <controls>\ttoolbar at left")
     print("\t-r, --right <controls>\ttoolbar at right")
@@ -696,10 +699,10 @@ a control with the name of the panel (see "MyPanel" below).
 
 For example:
 \t%s --embed \\
-\t-t "about,run,pause,quit" \\
-\t-p "MyPanel,Macro=Test,Undo=97,Help=98,>,F1=59,F2=60,F3=61,F4=62,>,close" \\
-\t-r "paste,debug,trace,machine,MyPanel" \\
-\t-b "sound,|,fastforward,|,fullscreen"
+\t--top "about,run,pause,quit" \\
+\t--panel "MyPanel,Macro=Test,Undo=97,Help=98,>,F1=59,F2=60,F3=61,F4=62,>,close" \\
+\t--right "MyPanel,debug,trace,machine" \\
+\t--bottom "sound,|,forward,|,full,|,quit"
 
 if no options are given, the UI uses basic controls.
 """ % name)
@@ -727,7 +730,11 @@ def main():
     for opt, arg in opts:
         print(opt, arg)
         if opt in ("-e", "--embed"):
-            embed = True
+            # GtkSocket window embedding needs both Python UI (Gtk3) and Hatari (SDL2) to run on X server
+            if "DISPLAY" in os.environ and "WAYLAND_DISPLAY" not in os.environ:
+                embed = True
+            else:
+                print("WARNING: ignoring embed option (Hatari window embedding works only under X11)")
         elif opt in ("-f", "--fullscreen"):
             fullscreen = True
         elif opt in ("-n", "--nomenu"):
