@@ -6,7 +6,7 @@
 
   A tiny graphical user interface for Hatari.
 */
-const char SDLGui_fileid[] = "Hatari sdlgui.c : " __DATE__ " " __TIME__;
+const char SDLGui_fileid[] = "Hatari sdlgui.c";
 
 #include <SDL.h>
 #include <ctype.h>
@@ -197,7 +197,7 @@ int SDLGui_SetScreen(SDL_Surface *pScrn)
 	if (sdlgui_fontheight < 16)
 		colors.underline = SDL_MapRGB(pSdlGuiScrn->format,255,0,255);
 	else
-		colors.underline = SDL_MapRGB(pSdlGuiScrn->format,0,0,0);		
+		colors.underline = SDL_MapRGB(pSdlGuiScrn->format,0,0,0);
 	colors.editfield = SDL_MapRGB(pSdlGuiScrn->format,160,160,160);
 
 	return 0;
@@ -540,7 +540,7 @@ static void SDLGui_DrawScrollbar(const SGOBJ *bdlg, int objnum)
 
 	x += bdlg[0].x*sdlgui_fontwidth;   /* add mainbox absolute coordinates */
 	y += bdlg[0].y*sdlgui_fontheight;  /* add mainbox absolute coordinates */
-	
+
 	w = 1 * sdlgui_fontwidth;
 	h = bdlg[objnum].w;
 
@@ -871,9 +871,9 @@ static void SDLGui_DebugPrintDialog(const SGOBJ *dlg)
 {
 #if DEBUG_INFO
 	int i;
-	printf("obj: flags | state\n");
+	printf("obj: flags | state\n");
 	for (i = 0; dlg[i].type != SGSTOP; i++)
-		printf("%3d:  0x%02x | 0x%02x\n", i, dlg[i].flags, dlg[i].state);
+		printf("%3d:  0x%02x | 0x%02x\n", i, dlg[i].flags, dlg[i].state);
 #endif
 }
 
@@ -1235,7 +1235,7 @@ int SDLGui_DoDialog(SGOBJ *dlg, SDL_Event *pEventOut, bool KeepCurrentObject)
 	while (retbutton == SDLGUI_NOTFOUND && !bQuitProgram)
 	{
 		if (SDL_WaitEvent(&sdlEvent) == 1)  /* Wait for events */
-		
+		{
 			switch (sdlEvent.type)
 			{
 			 case SDL_QUIT:
@@ -1330,12 +1330,28 @@ int SDLGui_DoDialog(SGOBJ *dlg, SDL_Event *pEventOut, bool KeepCurrentObject)
 				}
 				break;
 
+			 case SDL_JOYHATMOTION:
+				if (sdlEvent.jhat.value & SDL_HAT_LEFT)
+					retbutton = SDLGui_HandleShortcut(dlg, SG_SHORTCUT_LEFT);
+				else if (sdlEvent.jhat.value & SDL_HAT_RIGHT)
+					retbutton = SDLGui_HandleShortcut(dlg, SG_SHORTCUT_RIGHT);
+				if (sdlEvent.jhat.value & SDL_HAT_UP)
+				{
+					SDLGui_RemoveFocus(dlg, focused);
+					focused = SDLGui_FocusNext(dlg, focused, -1);
+				}
+				else if (sdlEvent.jhat.value & SDL_HAT_DOWN)
+				{
+					SDLGui_RemoveFocus(dlg, focused);
+					focused = SDLGui_FocusNext(dlg, focused, +1);
+				}
+				break;
+
 			 case SDL_JOYBUTTONDOWN:
 				retbutton = SDLGui_HandleSelection(dlg, focused, focused);
 				break;
 
 			 case SDL_JOYBALLMOTION:
-			 case SDL_JOYHATMOTION:
 			 case SDL_MOUSEMOTION:
 				break;
 
@@ -1418,6 +1434,7 @@ int SDLGui_DoDialog(SGOBJ *dlg, SDL_Event *pEventOut, bool KeepCurrentObject)
 					retbutton = SDLGUI_UNKNOWNEVENT;
 				break;
 			}
+		}
 	}
 
 	/* Restore background */
