@@ -6,7 +6,7 @@
 
   Low-level hard drive emulation
 */
-const char HDC_fileid[] = "Hatari hdc.c : " __DATE__ " " __TIME__;
+const char HDC_fileid[] = "Hatari hdc.c";
 
 #include <errno.h>
 #include <SDL_endian.h>
@@ -644,7 +644,7 @@ static void HDC_EmulateCommandPacket(SCSI_CTRLR *ctr)
  *   MIX     Minix partition, not supported by TOS
  *   SWP     Swap partition, not supported by TOS
  *   UNX     ASV (Atari System V) partition, not supported by TOS
- *   XGM     Extended partion
+ *   XGM     Extended partition
  *
  * Other partition types (listed in XHDI spec):
  *   MAC     MAC HFS partition, not supported by TOS
@@ -828,15 +828,18 @@ int HDC_InitDevice(SCSI_DEV *dev, char *filename, unsigned long blockSize)
 	if (filesize < 0)
 		return filesize;
 
-	fp = fopen(filename, "rb+");
-	if (fp == NULL)
+	if (!(fp = fopen(filename, "rb+")))
 	{
-		Log_Printf(LOG_ERROR, "Cannot open HD file read/write!\n");
-		return -ENOENT;
+		if (!(fp = fopen(filename, "rb")))
+		{
+			Log_AlertDlg(LOG_ERROR, "Cannot open HD file for reading\n'%s'!\n", filename);
+			return -ENOENT;
+		}
+		Log_AlertDlg(LOG_WARN, "HD file is read-only, no writes will go through\n'%s'.\n", filename);
 	}
-	if (!File_Lock(fp))
+	else if (!File_Lock(fp))
 	{
-		Log_Printf(LOG_ERROR, "Cannot lock HD file for writing!\n");
+		Log_AlertDlg(LOG_ERROR, "Locking HD file for writing failed\n'%s'!\n", filename);
 		fclose(fp);
 		return -ENOLCK;
 	}
@@ -850,7 +853,7 @@ int HDC_InitDevice(SCSI_DEV *dev, char *filename, unsigned long blockSize)
 }
 
 /**
- * Open the disk image file, set partitions.
+ * Open the disk image files, set partitions.
  */
 bool HDC_Init(void)
 {
@@ -874,19 +877,12 @@ bool HDC_Init(void)
 			continue;
 		if (HDC_InitDevice(&AcsiBus.devs[i], ConfigureParams.Acsi[i].sDeviceFile, ConfigureParams.Acsi[i].nBlockSize) == 0)
 		{
-			bAcsiEmuOn = true;
 			nAcsiPartitions += HDC_PartitionCount(AcsiBus.devs[i].image_file, TRACE_SCSI_CMD, NULL);
+			bAcsiEmuOn = true;
 		}
 	}
-
-	/* add SCSI partition count to ACSI ones
-	 * to support GEMDOS HD emu partition skipping
-	 */
-	nAcsiPartitions += Ncr5380_Init();
-
 	/* set total number of partitions */
 	nNumDrives += nAcsiPartitions;
-
 	return bAcsiEmuOn;
 }
 
@@ -912,10 +908,7 @@ void HDC_UnInit(void)
 	free(AcsiBus.buffer);
 	AcsiBus.buffer = NULL;
 
-	Ncr5380_UnInit();
-
-	if (bAcsiEmuOn)
-		nNumDrives -= nAcsiPartitions;
+	nNumDrives -= nAcsiPartitions;
 	nAcsiPartitions = 0;
 	bAcsiEmuOn = false;
 }
@@ -961,8 +954,8 @@ bool HDC_WriteCommandPacket(SCSI_CTRLR *ctr, Uint8 b)
 	++ctr->byteCount;
 
 	/* have we received a complete 6-byte class 0 or 10-byte class 1 packet yet? */
-	if ((ctr->opcode < 0x20 && ctr->byteCount >= 6) ||
-	    (ctr->opcode < 0x60 && ctr->byteCount >= 10))
+	if ((ctr->opcode < 0x20 && ctr->byteCount == 6) ||
+	    (ctr->opcode >= 0x20 && ctr->opcode < 0x60 && ctr->byteCount == 10))
 	{
 		/* We currently only support LUN 0, however INQUIRY must
 		 * always be handled, see SCSI standard */
@@ -987,8 +980,6 @@ bool HDC_WriteCommandPacket(SCSI_CTRLR *ctr, Uint8 b)
 				ctr->status = HD_STATUS_ERROR;
 			}
 		}
-
-		ctr->byteCount = 0;
 	}
 	else if (ctr->opcode >= 0x60)
 	{
