@@ -13,6 +13,7 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
+#include <inttypes.h>
 
 #include "main.h"
 #include "configuration.h"
@@ -44,9 +45,19 @@ static uint32_t ide_data_readw(void *opaque, uint32_t addr);
 static void ide_data_writel(void *opaque, uint32_t addr, uint32_t val);
 static uint32_t ide_data_readl(void *opaque, uint32_t addr);
 
-static bool Ide_MmioIsAvailable(void)
+/**
+ * Check whether IDE is available: The Falcon always has an IDE controller,
+ * and for the other machines it is normally only available on expansion
+ * cards - we assume that the users want us to emulate an IDE controller
+ * on such an expansion card if one of the IDE drives has been enabled.
+ * Note that we also disable IDE on Falcon if bFastBoot is enabled - TOS
+ * boots much faster if it does not have to scan for IDE devices.
+ */
+bool Ide_IsAvailable(void)
 {
-	return ConfigureParams.Ide[0].bUseDevice;
+	return ConfigureParams.Ide[0].bUseDevice ||
+	       ConfigureParams.Ide[1].bUseDevice ||
+	       (Config_IsMachineFalcon() && !ConfigureParams.System.bFastBoot);
 }
 
 /**
@@ -83,6 +94,8 @@ static uint32_t fcha2io(uint32_t address)
 
 /**
  * Handle byte read access from IDE IO memory.
+ * Note: Registers are available from usermode, too, so there is no check for
+ * the supervisor mode required here.
  */
 uae_u32 REGPARAM3 Ide_Mem_bget(uaecptr addr)
 {
@@ -92,10 +105,10 @@ uae_u32 REGPARAM3 Ide_Mem_bget(uaecptr addr)
 
 	addr &= 0x00ffffff;                           /* Use a 24 bit address */
 
-	if (addr >= 0xf00040 || !Ide_MmioIsAvailable())
+	if (addr >= 0xf00040 || !Ide_IsAvailable())
 	{
 		/* invalid memory addressing --> bus error */
-		M68000_BusError(addr_in, BUS_ERROR_READ, BUS_ERROR_SIZE_BYTE, BUS_ERROR_ACCESS_DATA);
+		M68000_BusError(addr_in, BUS_ERROR_READ, BUS_ERROR_SIZE_BYTE, BUS_ERROR_ACCESS_DATA, 0);
 		return -1;
 	}
 
@@ -129,10 +142,10 @@ uae_u32 REGPARAM3 Ide_Mem_wget(uaecptr addr)
 
 	addr &= 0x00ffffff;                           /* Use a 24 bit address */
 
-	if (addr >= 0xf00040 || !Ide_MmioIsAvailable())
+	if (addr >= 0xf00040 || !Ide_IsAvailable())
 	{
 		/* invalid memory addressing --> bus error */
-		M68000_BusError(addr_in, BUS_ERROR_READ, BUS_ERROR_SIZE_WORD, BUS_ERROR_ACCESS_DATA);
+		M68000_BusError(addr_in, BUS_ERROR_READ, BUS_ERROR_SIZE_WORD, BUS_ERROR_ACCESS_DATA, 0);
 		return -1;
 	}
 
@@ -160,10 +173,10 @@ uae_u32 REGPARAM3 Ide_Mem_lget(uaecptr addr)
 
 	addr &= 0x00ffffff;                           /* Use a 24 bit address */
 
-	if (addr >= 0xf00040 || !Ide_MmioIsAvailable())
+	if (addr >= 0xf00040 || !Ide_IsAvailable())
 	{
 		/* invalid memory addressing --> bus error */
-		M68000_BusError(addr_in, BUS_ERROR_READ, BUS_ERROR_SIZE_LONG, BUS_ERROR_ACCESS_DATA);
+		M68000_BusError(addr_in, BUS_ERROR_READ, BUS_ERROR_SIZE_LONG, BUS_ERROR_ACCESS_DATA, 0);
 		return -1;
 	}
 
@@ -186,6 +199,8 @@ uae_u32 REGPARAM3 Ide_Mem_lget(uaecptr addr)
 
 /**
  * Handle byte write access to IDE IO memory.
+ * Note: Registers are available from usermode, too, so there is no check for
+ * the supervisor mode required here.
  */
 void REGPARAM3 Ide_Mem_bput(uaecptr addr, uae_u32 val)
 {
@@ -197,10 +212,10 @@ void REGPARAM3 Ide_Mem_bput(uaecptr addr, uae_u32 val)
 
 	LOG_TRACE(TRACE_IDE, "IDE: bput($%x, $%x)\n", addr, val);
 
-	if (addr >= 0xf00040 || !Ide_MmioIsAvailable())
+	if (addr >= 0xf00040 || !Ide_IsAvailable())
 	{
 		/* invalid memory addressing --> bus error */
-		M68000_BusError(addr_in, BUS_ERROR_WRITE, BUS_ERROR_SIZE_BYTE, BUS_ERROR_ACCESS_DATA);
+		M68000_BusError(addr_in, BUS_ERROR_WRITE, BUS_ERROR_SIZE_BYTE, BUS_ERROR_ACCESS_DATA, val);
 		return;
 	}
 
@@ -229,10 +244,10 @@ void REGPARAM3 Ide_Mem_wput(uaecptr addr, uae_u32 val)
 
 	LOG_TRACE(TRACE_IDE, "IDE: wput($%x, $%x)\n", addr, val);
 
-	if (addr >= 0xf00040 || !Ide_MmioIsAvailable())
+	if (addr >= 0xf00040 || !Ide_IsAvailable())
 	{
 		/* invalid memory addressing --> bus error */
-		M68000_BusError(addr_in, BUS_ERROR_WRITE, BUS_ERROR_SIZE_WORD, BUS_ERROR_ACCESS_DATA);
+		M68000_BusError(addr_in, BUS_ERROR_WRITE, BUS_ERROR_SIZE_WORD, BUS_ERROR_ACCESS_DATA, val);
 		return;
 	}
 
@@ -254,10 +269,10 @@ void REGPARAM3 Ide_Mem_lput(uaecptr addr, uae_u32 val)
 
 	LOG_TRACE(TRACE_IDE, "IDE: lput($%x, $%x)\n", addr, val);
 
-	if (addr >= 0xf00040 || !Ide_MmioIsAvailable())
+	if (addr >= 0xf00040 || !Ide_IsAvailable())
 	{
 		/* invalid memory addressing --> bus error */
-		M68000_BusError(addr_in, BUS_ERROR_WRITE, BUS_ERROR_SIZE_LONG, BUS_ERROR_ACCESS_DATA);
+		M68000_BusError(addr_in, BUS_ERROR_WRITE, BUS_ERROR_SIZE_LONG, BUS_ERROR_ACCESS_DATA, val);
 		return;
 	}
 
@@ -463,7 +478,8 @@ static int bdrv_read(BlockDriverState *bs, int64_t sector_num,
 	ret = fread(buf, 1, len, bs->fhndl);
 	if (ret != len)
 	{
-		fprintf(stderr,"IDE: bdrv_read error (%d != %d length) at sector %lu!\n", ret, len, (unsigned long)sector_num);
+		Log_Printf(LOG_ERROR, "IDE: bdrv_read error (%d != %d length) at sector %lu!\n",
+		           ret, len, (unsigned long)sector_num);
 		return -EINVAL;
 	}
 
@@ -527,8 +543,8 @@ static int bdrv_write(BlockDriverState *bs, int64_t sector_num,
 	}
 	if (ret != len)
 	{
-		fprintf(stderr,"IDE: bdrv_write error (%d != %d length) at sector %lu!\n",
-		        ret, len,  (unsigned long)sector_num);
+		Log_Printf(LOG_ERROR, "IDE: bdrv_write error (%d != %d length) at sector %lu!\n",
+		           ret, len,  (unsigned long)sector_num);
 		return -EIO;
 	}
 
@@ -847,7 +863,7 @@ static void bdrv_eject(BlockDriverState *bs, int eject_flag)
 #define GPMODE_TO_PROTECT_PAGE		0x1d
 #define GPMODE_CAPABILITIES_PAGE	0x2a
 #define GPMODE_ALL_PAGES		0x3f
-/* Not in Mt. Fuji, but in ATAPI 2.6 -- depricated now in favor
+/* Not in Mt. Fuji, but in ATAPI 2.6 -- deprecated now in favor
  * of MODE_SENSE_POWER_PAGE */
 #define GPMODE_CDROM_PAGE		0x0d
 
@@ -983,7 +999,8 @@ static void ide_identify(IDEState *s)
 	padstr((char *)(p + 23), FW_VERSION, 8); /* firmware version */
 	/* Use the same convention for the name as SCSI disks are using: The
 	 * first 8 characters should be the vendor, i.e. use 2 spaces here */
-	snprintf(buf, sizeof(buf), "Hatari  IDE disk %liM", (long)(s->nb_sectors / 2048));
+	snprintf(buf, sizeof(buf), "Hatari  IDE disk %liM",
+	         (long)(s->nb_sectors / (1024 * 1024 / s->bs->sector_size)));
 	padstr((char *)(p + 27), buf, 40);
 #if MAX_MULT_SECTORS > 1
 	put_le16(p + 47, 0x8000 | MAX_MULT_SECTORS);
@@ -1120,7 +1137,7 @@ static inline void ide_set_irq(IDEState *s)
 	if (!(s->cmd & IDE_CMD_DISABLE_IRQ))
 	{
 		/* Set IRQ (set line to low) */
-		MFP_GPIP_Set_Line_Input ( MFP_GPIP_LINE_FDC_HDC , MFP_GPIP_STATE_LOW );
+		MFP_GPIP_Set_Line_Input ( pMFP_Main , MFP_GPIP_LINE_FDC_HDC , MFP_GPIP_STATE_LOW );
 	}
 }
 
@@ -1692,7 +1709,6 @@ static void ide_atapi_cmd(IDEState *s)
 		uint64_t total_sectors;
 
 		bdrv_get_geometry(s->bs, &total_sectors);
-		total_sectors >>= 2;
 		if (total_sectors == 0)
 		{
 			ide_atapi_cmd_error(s, SENSE_NOT_READY,
@@ -1748,7 +1764,6 @@ static void ide_atapi_cmd(IDEState *s)
 		uint64_t total_sectors;
 
 		bdrv_get_geometry(s->bs, &total_sectors);
-		total_sectors >>= 2;
 		if (total_sectors == 0)
 		{
 			ide_atapi_cmd_error(s, SENSE_NOT_READY,
@@ -1762,7 +1777,7 @@ static void ide_atapi_cmd(IDEState *s)
 		switch (format)
 		{
 		case 0:
-			fprintf(stderr,"IDE FIXME: cdrom_read_toc");
+			Log_Printf(LOG_ERROR, "IDE FIXME: cdrom_read_toc not implemented");
 			len=-1;
 			//len = cdrom_read_toc(total_sectors, buf, msf, start_track);
 			if (len < 0)
@@ -1778,7 +1793,7 @@ static void ide_atapi_cmd(IDEState *s)
 			ide_atapi_cmd_reply(s, 12, max_len);
 			break;
 		case 2:
-			fprintf(stderr,"IDE FIXME: cdrom_read_toc_raw");
+			Log_Printf(LOG_ERROR, "IDE FIXME: cdrom_read_toc_raw not implemented");
 			len=-1;
 			//len = cdrom_read_toc_raw(total_sectors, buf, msf, start_track);
 			if (len < 0)
@@ -1798,7 +1813,6 @@ error_cmd:
 		uint64_t total_sectors;
 
 		bdrv_get_geometry(s->bs, &total_sectors);
-		total_sectors >>= 2;
 		if (total_sectors == 0)
 		{
 			ide_atapi_cmd_error(s, SENSE_NOT_READY,
@@ -1828,7 +1842,6 @@ error_cmd:
 		{
 		case 0:
 			bdrv_get_geometry(s->bs, &total_sectors);
-			total_sectors >>= 2;
 			if (total_sectors == 0)
 			{
 				ide_atapi_cmd_error(s, SENSE_NOT_READY,
@@ -2022,7 +2035,8 @@ static void ide_ioport_write(void *opaque, uint32_t addr, uint32_t val)
 		/* ignore commands to non existent IDE device 1 */
 		if (s != ide_if && !s->bs)
 		{
-			fprintf(stderr,"IDE: CMD to non-existant IDE device #1!\n");
+			Log_Printf(LOG_INFO, "IDE: Tried to send command to "
+			           "non-existent IDE device #1!\n");
 			break;
 		}
 
@@ -2147,7 +2161,7 @@ static void ide_ioport_write(void *opaque, uint32_t addr, uint32_t val)
 				goto abort_cmd;
 			ide_cmd_lba48_transform(s, lba48);
 			// ide_sector_read_dma(s);
-			fprintf(stderr, "IDE: DMA read not supported!\n");
+			Log_Printf(LOG_ERROR, "IDE: DMA read not supported!\n");
 			break;
 		case WIN_WRITEDMA_EXT:
 			lba48 = 1;
@@ -2158,7 +2172,7 @@ static void ide_ioport_write(void *opaque, uint32_t addr, uint32_t val)
 				goto abort_cmd;
 			ide_cmd_lba48_transform(s, lba48);
 			// ide_sector_write_dma(s);
-			fprintf(stderr, "IDE: DMA write not supported!\n");
+			Log_Printf(LOG_ERROR, "IDE: DMA write not supported!\n");
 			s->media_changed = 1;
 			break;
 		case WIN_READ_NATIVE_MAX_EXT:
@@ -2369,7 +2383,7 @@ static uint32_t ide_ioport_read(void *opaque, uint32_t addr1)
 			ret = s->status;
 
 		/* Clear IRQ (set line to high) */
-		MFP_GPIP_Set_Line_Input ( MFP_GPIP_LINE_FDC_HDC , MFP_GPIP_STATE_HIGH );
+		MFP_GPIP_Set_Line_Input ( pMFP_Main , MFP_GPIP_LINE_FDC_HDC , MFP_GPIP_STATE_HIGH );
 		break;
 	}
 	LOG_TRACE(TRACE_IDE, "IDE: read addr=0x%x val=%02x\n", addr1, ret);
@@ -2436,6 +2450,9 @@ static void ide_data_writew(void *opaque, uint32_t addr, uint32_t val)
 	IDEState *s = ((IDEState *)opaque)->cur_drive;
 	uint8_t *p;
 
+	if (!s->data_ptr || s->data_ptr > s->data_end)
+		return;
+
 	p = s->data_ptr;
 	*(uint16_t *)p = le16_to_cpu(val);
 	p += 2;
@@ -2449,6 +2466,10 @@ static uint32_t ide_data_readw(void *opaque, uint32_t addr)
 	IDEState *s = ((IDEState *)opaque)->cur_drive;
 	uint8_t *p;
 	int ret;
+
+	if (!s->data_ptr || s->data_ptr > s->data_end)
+		return 0xffff;
+
 	p = s->data_ptr;
 	ret = cpu_to_le16(*(uint16_t *)p);
 	p += 2;
@@ -2463,6 +2484,9 @@ static void ide_data_writel(void *opaque, uint32_t addr, uint32_t val)
 	IDEState *s = ((IDEState *)opaque)->cur_drive;
 	uint8_t *p;
 
+	if (!s->data_ptr || s->data_ptr > s->data_end)
+		return;
+
 	p = s->data_ptr;
 	*(uint32_t *)p = le32_to_cpu(val);
 	p += 4;
@@ -2475,7 +2499,10 @@ static uint32_t ide_data_readl(void *opaque, uint32_t addr)
 {
 	IDEState *s = ((IDEState *)opaque)->cur_drive;
 	uint8_t *p;
-	int ret;
+	uint32_t ret;
+
+	if (!s->data_ptr || s->data_ptr > s->data_end)
+		return 0xffffffff;
 
 	p = s->data_ptr;
 	ret = cpu_to_le32(*(uint32_t *)p);
@@ -2567,8 +2594,6 @@ static int guess_disk_lchs(IDEState *s,
 			*pheads = heads;
 			*psectors = sectors;
 			*pcylinders = cylinders;
-			LOG_TRACE(TRACE_IDE, "IDE: guessed geometry LCHS=%d %d %d\n",
-			       cylinders, heads, sectors);
 			free(buf);
 			return 0;
 		}
@@ -2588,6 +2613,11 @@ static void ide_init2(IDEState *ide_state, BlockDriverState *hd0,
 	for (i = 0; i < 2; i++)
 	{
 		s = ide_state + i;
+		s->cur_drive = s;
+
+		if (!ConfigureParams.Ide[i].bUseDevice)
+			continue;
+
 		s->io_buffer = malloc(MAX_MULT_SECTORS * MAX_SECTOR_SIZE + 4);
 		assert(s->io_buffer);
 		if (i == 0)
@@ -2661,6 +2691,8 @@ default_geometry:
 				}
 				bdrv_set_geometry_hint(s->bs, s->cylinders, s->heads, s->sectors);
 			}
+			LOG_TRACE(TRACE_IDE, "IDE: using geometry LCHS=%d %d %d for drive %d\n",
+			       s->cylinders, s->heads, s->sectors, i);
 			if (bdrv_get_type_hint(s->bs) == BDRV_TYPE_CDROM)
 			{
 				s->is_cdrom = 1;
@@ -2687,7 +2719,7 @@ void Ide_Init(void)
 {
 	int i;
 
-	if (!ConfigureParams.Ide[0].bUseDevice)
+	if (!Ide_IsAvailable() )
 		return;
 
 	opaque_ide_if = calloc(2, sizeof(IDEState));
@@ -2709,7 +2741,8 @@ void Ide_Init(void)
 				hd_table[i]->byteswap = !is_byteswap;
 			else
 				hd_table[i]->byteswap = !ConfigureParams.Ide[i].nByteSwap;
-
+			LOG_TRACE(TRACE_IDE, "IDE: little->big endian byte-swapping %s for drive %d\n",
+				  hd_table[i]->byteswap ? "enabled" : "disabled", i);
 			hd_table[i]->sector_size = ConfigureParams.Ide[i].nBlockSize;
 			hd_table[i]->type = ConfigureParams.Ide[i].nDeviceType;
 		}
