@@ -28,9 +28,13 @@ const char NCR5380_fileid[] = "Hatari ncr5380.c";
 #include "ncr5380.h"
 #include "stMemory.h"
 #include "newcpu.h"
+#include "tos.h"
 
 #define WITH_NCR5380 1
 
+int nScsiPartitions;
+bool bScsiEmuOn;
+
 static SCSI_CTRLR ScsiBus;
 
 #define MAX_TOTAL_SCSI_DEVICES 8
@@ -116,6 +120,7 @@ static const uae_s16 scsicmdsizes[] = { 6, 10, 10, 12, 16, 12, 10, 6 };
 
 static uae_u8 ncr5380_bget(struct soft_scsi *scsi, int reg);
 void ncr5380_bput(struct soft_scsi *scsi, int reg, uae_u8 v);
+static void ncr5380_set_irq(struct soft_scsi *scsi);
 
 static int scsi_data_dir(struct scsi_data *sd)
 {
@@ -756,14 +761,9 @@ static void dma_check(struct soft_scsi *ncr)
 	}
 
 	if (Config_IsMachineFalcon())
-	{
-		FDC_SetDMAStatus(ScsiBus.bDmaError);	/* Mark DMA error */
-		FDC_SetIRQ(FDC_IRQ_SOURCE_HDC);
-	}
-	else
-	{
-		ncr->irq = true;
-	}
+		FDC_SetDMAStatus(ScsiBus.bDmaError);	/* Set/Unset DMA error */
+
+	ncr5380_set_irq ( ncr );
 
 	if (ScsiBus.offset == ScsiBus.data_len)
 	{
@@ -788,6 +788,8 @@ static void ncr5380_set_irq(struct soft_scsi *scsi)
 
 	if (Config_IsMachineFalcon())
 		FDC_SetIRQ(FDC_IRQ_SOURCE_HDC);
+	else if (Config_IsMachineTT())
+		MFP_GPIP_Set_Line_Input ( pMFP_TT , MFP_TT_GPIP_LINE_SCSI_NCR , MFP_GPIP_STATE_HIGH );
 }
 
 static void ncr5380_databusoutput(struct soft_scsi *scsi)
@@ -1010,12 +1012,16 @@ void ncr5380_bput(struct soft_scsi *scsi, int reg, uae_u8 v)
 /* ***** Hatari glue code below ***** */
 
 /**
- * return number of identified partitions on existing SCSI images
+ * Open the disk image file, set partitions count.
+ * Return true if there are any.
  */
-int Ncr5380_Init(void)
+bool Ncr5380_Init(void)
 {
 #if WITH_NCR5380
-	int i, partitions = 0;
+	int i;
+
+	nScsiPartitions = 0;
+	bScsiEmuOn = false;
 
 	memset(&ScsiBus, 0, sizeof(ScsiBus));
 	ScsiBus.typestr = "SCSI";
@@ -1031,14 +1037,20 @@ int Ncr5380_Init(void)
 		if (!ConfigureParams.Scsi[i].bUseDevice)
 			continue;
 		if (HDC_InitDevice(&ScsiBus.devs[i], ConfigureParams.Scsi[i].sDeviceFile, ConfigureParams.Scsi[i].nBlockSize) == 0)
-			partitions += HDC_PartitionCount(ScsiBus.devs[i].image_file, TRACE_SCSI_CMD, NULL);
+		{
+			nScsiPartitions += HDC_PartitionCount(ScsiBus.devs[i].image_file, TRACE_SCSI_CMD, NULL);
+			bScsiEmuOn = true;
+		}
 	}
-	return partitions;
-#else
-	return 0;
+	nNumDrives += nScsiPartitions;
 #endif
+	return bScsiEmuOn;
 }
 
+/**
+ * Ncr5380_UnInit - close image files and free resources
+ *
+ */
 void Ncr5380_UnInit(void)
 {
 #if WITH_NCR5380
@@ -1055,6 +1067,10 @@ void Ncr5380_UnInit(void)
 	}
 	free(ScsiBus.buffer);
 	ScsiBus.buffer = NULL;
+
+	nNumDrives -= nScsiPartitions;
+	nScsiPartitions = 0;
+	bScsiEmuOn = false;
 #endif
 }
 
@@ -1148,16 +1164,3 @@ void Ncr5380_TT_DMA_Ctrl_WriteWord(void)
 }
 
 
-/**
- * This is a temporary hack until we've got proper emulation of the
- * 2nd MFP in the TT
- */
-void Ncr5380_TT_GPIP_ReadByte(void)
-{
-	Uint8 val = 0x7f;
-
-	if (ncr_soft_scsi.irq)
-		val |= 0x80;
-
-	IoMem[0xfffa81] = val;
-}
