@@ -24,7 +24,7 @@
   Tyranny by DHS).
 */
 
-const char Screen_fileid[] = "Hatari screen.c : " __DATE__ " " __TIME__;
+const char Screen_fileid[] = "Hatari screen.c";
 
 #include <SDL.h>
 #include <SDL_endian.h>
@@ -33,6 +33,7 @@ const char Screen_fileid[] = "Hatari screen.c : " __DATE__ " " __TIME__;
 #include "main.h"
 #include "configuration.h"
 #include "avi_record.h"
+#include "file.h"
 #include "log.h"
 #include "paths.h"
 #include "options.h"
@@ -108,12 +109,16 @@ SDL_Window *sdlWindow;
 static SDL_Renderer *sdlRenderer;
 static SDL_Texture *sdlTexture;
 static bool bUseSdlRenderer;            /* true when using SDL2 renderer */
+static bool bIsSoftwareRenderer;
 
 void SDL_UpdateRects(SDL_Surface *screen, int numrects, SDL_Rect *rects)
 {
 	if (bUseSdlRenderer)
 	{
 		SDL_UpdateTexture(sdlTexture, NULL, screen->pixels, screen->pitch);
+		/* Need to clear the renderer context for certain accelerated cards */
+		if (!bIsSoftwareRenderer)
+			SDL_RenderClear(sdlRenderer);
 		SDL_RenderCopy(sdlRenderer, sdlTexture, NULL, NULL);
 		SDL_RenderPresent(sdlRenderer);
 	}
@@ -311,6 +316,77 @@ static void Screen_FreeSDL2Resources(void)
 		sdlRenderer = NULL;
 	}
 }
+
+/*
+ * Create window backing texture when needed, with suitable scaling
+ * quality.
+ *
+ * Window size is affected by ZoomFactor setting and window resizes
+ * done by the user, and constrained by maximum window size setting
+ * and desktop size.
+ *
+ * Calculate scale factor for the given resulting window size, compared
+ * to the size of the SDL frame buffer rendered by Hatari, and based on
+ * that, set the render scaling quality hint to:
+ * - (sharp) nearest pixel sampling for integer zoom factors
+ * - (smoothing/blurring) linear filtering otherwise
+ *
+ * If hint value changes from earlier one (or force flag is used),
+ * window texture needs to be re-created to apply the scaling quality
+ * change.
+ */
+void Screen_SetTextureScale(int width, int height, int win_width, int win_height, bool bForce)
+{
+	static char prev_quality = '0';
+	float scale_w, scale_h, scale;
+	char quality;
+	int pfmt;
+
+	if (!(bUseSdlRenderer && sdlRenderer))
+		return;
+
+	scale_w = (float)win_width / width;
+	scale_h = (float)win_height / height;
+	scale = (scale_w + scale_h) / 2.0;
+
+	if (scale == floorf(scale))
+		quality = '0';	// nearest pixel
+	else
+		quality = '1';	// linear filtering
+
+	DEBUGPRINT(("%dx%d / %dx%d -> scale = %.1f / Render Scale Quality = %c\n",
+		    win_width, win_height, width, height, scale, quality));
+
+	if (bForce || quality != prev_quality)
+	{
+		char hint[2] = { quality, 0 };
+		prev_quality = quality;
+
+		/* hint needs to be there before texture */
+		SDL_SetHintWithPriority(SDL_HINT_RENDER_SCALE_QUALITY, hint, SDL_HINT_OVERRIDE);
+
+		if (sdlTexture)
+		{
+			SDL_DestroyTexture(sdlTexture);
+			sdlTexture = NULL;
+		}
+
+		if (sdlscrn->format->BitsPerPixel == 16)
+			pfmt = SDL_PIXELFORMAT_RGB565;
+		else
+			pfmt = SDL_PIXELFORMAT_RGB888;
+
+		sdlTexture = SDL_CreateTexture(sdlRenderer, pfmt,
+					       SDL_TEXTUREACCESS_STREAMING,
+					       width, height);
+		if (!sdlTexture)
+		{
+			fprintf(stderr, "ERROR: Failed to create %dx%d@%d texture!\n",
+			       width, height, sdlscrn->format->BitsPerPixel);
+			exit(-3);
+		}
+	}
+}
 #endif
 
 /**
@@ -323,10 +399,10 @@ bool Screen_SetSDLVideoSize(int width, int height, int bitdepth, bool bForceChan
 	char *psSdlVideoDriver;
 	bool bUseDummyMode;
 #if WITH_SDL2
-	static int nPrevRenderScaleQuality = 0;
 	static bool bPrevUseVsync = false;
 	static bool bPrevInFullScreen;
 	int win_width, win_height;
+	float scale = 1.0;
 
 	if (bitdepth == 0 || bitdepth == 24)
 		bitdepth = 32;
@@ -338,17 +414,13 @@ bool Screen_SetSDLVideoSize(int width, int height, int bitdepth, bool bForceChan
 
 	psSdlVideoDriver = SDL_getenv("SDL_VIDEODRIVER");
 	bUseDummyMode = psSdlVideoDriver && !strcmp(psSdlVideoDriver, "dummy");
+#if !WITH_SDL2
 	if (bUseDummyMode)
 	{
 		ConfigureParams.Screen.DisableVideo = true;
 	}
-
-#ifdef _MUDFLAP
-	if (sdlscrn)
-	{
-		__mf_unregister(sdlscrn->pixels, sdlscrn->pitch*sdlscrn->h, __MF_TYPE_GUESS);
-	}
 #endif
+
 	if (bInFullScreen)
 	{
 		/* unhide the Hatari WM window for fullscreen */
@@ -361,6 +433,12 @@ bool Screen_SetSDLVideoSize(int width, int height, int bitdepth, bool bForceChan
 	/* SDL Video attributes: */
 	win_width = width;
 	win_height = height;
+	if (bUseSdlRenderer)
+	{
+		scale = ConfigureParams.Screen.nZoomFactor;
+		win_width *= scale;
+		win_height *= scale;
+	}
 	if (bInFullScreen)
 	{
 		sdlVideoFlags = SDL_WINDOW_BORDERLESS | SDL_WINDOW_INPUT_GRABBED;
@@ -398,13 +476,6 @@ bool Screen_SetSDLVideoSize(int width, int height, int bitdepth, bool bForceChan
 	}
 	bPrevInFullScreen = bInFullScreen;
 
-	/* Set SDL2 video hints */
-	if (nPrevRenderScaleQuality != ConfigureParams.Screen.nRenderScaleQuality)
-	{
-		char hint[2] = { '0' + ConfigureParams.Screen.nRenderScaleQuality, 0 };
-		SDL_SetHintWithPriority(SDL_HINT_RENDER_SCALE_QUALITY, hint, SDL_HINT_OVERRIDE);
-		nPrevRenderScaleQuality = ConfigureParams.Screen.nRenderScaleQuality;
-	}
 	if (bPrevUseVsync != ConfigureParams.Screen.bUseVsync)
 	{
 		char hint[2] = { '0' + ConfigureParams.Screen.bUseVsync, 0 };
@@ -418,8 +489,8 @@ bool Screen_SetSDLVideoSize(int width, int height, int bitdepth, bool bForceChan
 #endif
 
 	/* Set new video mode */
-	DEBUGPRINT(("SDL screen request: %d x %d @ %d (%s)\n", width, height,
-	        bitdepth, bInFullScreen?"fullscreen":"windowed"));
+	DEBUGPRINT(("SDL screen request: %d x %d @ %d (%s) -> window: %d x %d\n", width, height,
+	        bitdepth, (bInFullScreen ? "fullscreen" : "windowed"), win_width, win_height));
 
 	if (sdlWindow)
 	{
@@ -440,7 +511,8 @@ bool Screen_SetSDLVideoSize(int width, int height, int bitdepth, bool bForceChan
 	}
 	if (bUseSdlRenderer)
 	{
-		int rm, bm, gm, pfmt;
+		int rm, bm, gm;
+		SDL_RendererInfo sRenderInfo = { 0 };
 
 		sdlRenderer = SDL_CreateRenderer(sdlWindow, -1, 0);
 		if (!sdlRenderer)
@@ -452,36 +524,38 @@ bool Screen_SetSDLVideoSize(int width, int height, int bitdepth, bool bForceChan
 
 		if (bInFullScreen)
 			SDL_RenderSetLogicalSize(sdlRenderer, width, height);
+		else
+			SDL_RenderSetScale(sdlRenderer, scale, scale);
+
+		/* Force to black to stop side bar artifacts on 16:9 monitors. */
+		SDL_SetRenderDrawColor(sdlRenderer, 0, 0, 0, 255);
+		SDL_RenderClear(sdlRenderer);
+		SDL_RenderPresent(sdlRenderer);
+
+		SDL_GetRendererInfo(sdlRenderer, &sRenderInfo);
+		bIsSoftwareRenderer = sRenderInfo.flags & SDL_RENDERER_SOFTWARE;
 
 		if (bitdepth == 16)
 		{
 			rm = 0xF800;
 			gm = 0x07E0;
 			bm = 0x001F;
-			pfmt = SDL_PIXELFORMAT_RGB565;
 		}
 		else
 		{
 			rm = 0x00FF0000;
 			gm = 0x0000FF00;
 			bm = 0x000000FF;
-			pfmt = SDL_PIXELFORMAT_RGB888;
 		}
 		sdlscrn = SDL_CreateRGBSurface(0, width, height, bitdepth,
 		                               rm, gm, bm, 0);
-		sdlTexture = SDL_CreateTexture(sdlRenderer, pfmt,
-		                               SDL_TEXTUREACCESS_STREAMING,
-		                               width, height);
-		if (!sdlTexture)
-		{
-			fprintf(stderr, "ERROR: Failed to create %dx%d@%d texture!\n",
-			       width, height, bitdepth);
-			exit(-3);
-		}
+
+		Screen_SetTextureScale(width, height, win_width, win_height, true);
 	}
 	else
 	{
 		sdlscrn = SDL_GetWindowSurface(sdlWindow);
+		bIsSoftwareRenderer = true;
 	}
 
 #else	/* !WITH_SDL2 */
@@ -528,10 +602,6 @@ bool Screen_SetSDLVideoSize(int width, int height, int bitdepth, bool bForceChan
 	DEBUGPRINT(("SDL screen granted: %d x %d @ %d\n", sdlscrn->w, sdlscrn->h,
 	            sdlscrn->format->BitsPerPixel));
 
-#ifdef _MUDFLAP
-	__mf_register(sdlscrn->pixels, sdlscrn->pitch*sdlscrn->h, __MF_TYPE_GUESS, "SDL pixels");
-#endif
-
 	if (!bInFullScreen)
 	{
 		/* re-embed the new Hatari SDL window */
@@ -750,8 +820,8 @@ void Screen_Init(void)
 	Video_SetScreenRasters();                       /* Set rasters ready for first screen */
 
 	/* Load and set icon */
-	snprintf(sIconFileName, sizeof(sIconFileName), "%s%chatari-icon.bmp",
-	         Paths_GetDataDir(), PATHSEP);
+	File_MakePathBuf(sIconFileName, sizeof(sIconFileName), Paths_GetDataDir(),
+	                 "hatari-icon", "bmp");
 	pIconSurf = SDL_LoadBMP(sIconFileName);
 	if (pIconSurf)
 	{
@@ -1600,6 +1670,73 @@ static void Convert_StartFrame(void)
 		AdjustLinePaletteRemap(y++);     /* Update palette */
 }
 
+/*-----------------------------------------------------------------------*/
+/**
+ * Copy given line (address) of given length, to line below it
+ * as-is, or halve its intensity, depending on bScrDoubleY.
+ *
+ * Source line is already in host format, so we don't need to
+ * care about endianness.
+ *
+ * Return address to next line after the copy
+ */
+static Uint32* Double_ScreenLine32(Uint32 *line, int size)
+{
+	SDL_PixelFormat *fmt;
+	int fmt_size = size/4;
+	Uint32 *next;
+	Uint32 mask;
+
+	next = line + fmt_size;
+	/* copy as-is */
+	if (bScrDoubleY)
+	{
+		memcpy(next, line, size);
+		return next + fmt_size;
+	}
+	/* TV-mode -- halve the intensity while copying */
+	fmt = sdlscrn->format;
+	mask = ((fmt->Rmask >> 1) & fmt->Rmask)
+	     | ((fmt->Gmask >> 1) & fmt->Gmask)
+	     | ((fmt->Bmask >> 1) & fmt->Bmask);
+	do {
+		*next++ = (*line++ >> 1) & mask;
+	}
+	while (--fmt_size);
+
+	return next;
+}
+
+/**
+ * 16-bit variant of Double_ScreenLine32()
+ */
+static Uint16* Double_ScreenLine16(Uint16 *line, int size)
+{
+	SDL_PixelFormat *fmt;
+	int fmt_size = size/2;
+	Uint16 *next;
+	Uint16 mask;
+
+	next = line + fmt_size;
+	/* copy as-is */
+	if (bScrDoubleY)
+	{
+		memcpy(next, line, size);
+		return next + fmt_size;
+	}
+	/* TV-mode -- halve the intensity while copying */
+	fmt = sdlscrn->format;
+	mask = ((fmt->Rmask >> 1) & fmt->Rmask)
+	     | ((fmt->Gmask >> 1) & fmt->Gmask)
+	     | ((fmt->Bmask >> 1) & fmt->Bmask);
+	do {
+		*next++ = (*line++ >> 1) & mask;
+	}
+	while (--fmt_size);
+
+	return next;
+}
+
 /* lookup tables and conversion macros */
 #include "convert/macros.h"
 
